﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Drawing;                 //【Bitmap】【Color】
using System.Drawing.Imaging;         //【BitmapData】
using System.Runtime.InteropServices; //【Marshal.Copy】

namespace Bitmap_GetDepthImage
{
    class NTL_ColorBar
    {
        public Color[] NTL_ColorPallet;
        public Int32[] NTL_ColorCode;
        public byte[] NTL_ColorElement;
        public Bitmap bmp;

        //*******************************************************
        //【CreateColorPallet】ColorPalletを初期化してｎ色分の領域を確保する
        //*******************************************************
        public void CreateColorPallet(int n)
        {
           NTL_ColorPallet = new Color[n];
           NTL_ColorCode = new Int32[n];
           NTL_ColorElement = new byte[n*4];
        }

        //*******************************************************
        //【GenerateColor】指定パレット番号i1からi2までを指定色P1からP2まで直線補間した色を作って設定する
        //*******************************************************
        public void GenerateColor(int i1, Color C1, int i2, Color C2)
        {
            float A, R, G, B, dA, dR, dG, dB, d;
            d=1.0f/((float)i2-(float)i1);
            A=(float)C1.A;
            R=(float)C1.R;
            G=(float)C1.G;
            B=(float)C1.B;
            dA =((float)C2.A-A)*d;
            dR =((float)C2.R-R)*d;
            dG =((float)C2.G-G)*d;
            dB =((float)C2.B-B)*d;
            for(int i=i1; i<=i2; i++)
            {
                NTL_ColorPallet[i]=Color.FromArgb((int)A,(int)R,(int)G,(int)B);
                NTL_ColorCode[i]=(int)((long)A<<24 | (long)R<<16 | (long)G<<8 | (long)B);
                NTL_ColorElement[4*i  ]=(byte)B;
                NTL_ColorElement[4*i+1]=(byte)G;
                NTL_ColorElement[4*i+2]=(byte)R;
                NTL_ColorElement[4*i+3]=(byte)A;
                A+=dA; R+=dR; G+=dG; B+=dB;
            }
        }

        //*******************************************************
        //【CreateColorPalletForAbsolute】絶対値表示用のカラーパレットを作る
        //*******************************************************
        public void CreateColorPalletForAbsolute(int n)
        {
            //  白⇒赤⇒橙⇒黄⇒緑⇒淡青⇒青⇒藍⇒紫⇒茶⇒淡灰⇒ 灰⇒濃灰⇒黒
            Color[] C = new Color[] { Color.White, Color.Red, Color.Orange, Color.Yellow, Color.Green, Color.Aqua, Color.Blue,
            Color.DarkBlue, Color.Violet, Color.Brown, Color.LightGray, Color.Gray, Color.DarkGray, Color.Black};
            int[] index = new int[14];
            index[0] = 0; index[13] = (int)(n - 1);
            CreateColorPallet(n); //【ｎ色分のカラーパレットを作成】
            for(int i=1; i<13; i++) { index[i]=(int)(i * (float)n / 13.0f); }
            for (int i = 0; i < 13; i++) { GenerateColor(index[i], C[i], index[i+1], C[i+1]); }
        }

        //*******************************************************
        //【CreateColorPallet】指定配列を使ってカラーパレットを作る
        //*******************************************************
        public void CreateColorPalletForAbsolute(int n, int[] index, Color[] c)
        {
            CreateColorPallet(n); //【ｎ色分のカラーパレットを作成】
            for (int i = 0; i < n - 1; i++)
            {
                GenerateColor(index[i], c[i], index[i+1], c[i+1]);
            }
        }

        //*******************************************************
        //【GetColor】３通りの型でパレット番号を色に変換する
        //*******************************************************
        public Color GetColor(int a)
        {
            return NTL_ColorPallet[a];
        }

        public int GetColorCode(int a)
        {
            return NTL_ColorCode[a];
        }

        public void GetColorElement(int a, out int A, out int R, out int G, out int B)
        {
            B = NTL_ColorElement[4 * a];
            G = NTL_ColorElement[4 * a + 1];
            R = NTL_ColorElement[4 * a + 2];
            A = NTL_ColorElement[4 * a + 3];
        }

        //*******************************************************
        //【GetPalletSize】パレットの色数を得る
        //*******************************************************
        public int GetPalletSize
        {
            get { return NTL_ColorCode.Length; }
        }

        //*******************************************************
        //【CreateColorScale】Bitmap形式のカラースケールを作る
        //*******************************************************
        public Bitmap CreateColorScale(int width, int height)
        {
            int i,j,k,a,r,g,b,c; float d;
            Bitmap bmp1 = new Bitmap(width, height, System.Drawing.Imaging.PixelFormat.Format32bppPArgb); //【ColorImage表示用】
            Rectangle rect1 = new Rectangle(0, 0, width, height); //【ColorImage用枠】
            byte[] pixelData = new byte[width*height*4];
            d = (float)NTL_ColorCode.Length/(float)width;
            for (i = 0; i < width; i++)
            {
                c = (int)((float)i * d)*4;
                b = NTL_ColorElement[c];
                g = NTL_ColorElement[c + 1];
                r = NTL_ColorElement[c + 2];
                a = NTL_ColorElement[c + 3];
                for (j = 0; j < height; j++)
                {
                    k = (j * width + i) * 4;
                    pixelData[k    ] = (byte)b;
                    pixelData[k + 1] = (byte)g;
                    pixelData[k + 2] = (byte)r;
                    pixelData[k + 3] = (byte)a;
                }
            }
            BitmapData data1 = bmp1.LockBits(rect1, ImageLockMode.WriteOnly, System.Drawing.Imaging.PixelFormat.Format32bppPArgb); //【Lock】
            Marshal.Copy(pixelData, 0, data1.Scan0, pixelData.Length); //【UnmanagedMemoryのBitmapDataに配列PixelDataをコピー】
            bmp1.UnlockBits(data1);           //【Lock解除】
        //  bmp1.RotateFlip(RotateFlipType.Rotate180FlipY); } //【※もし左右反転とか回転とかするなら】
        //  bmp1.RotateFlip(RotateFlipType.Rotate90FlipX); }  //
            return bmp1;
        }
    }
}
